/**
 * axios实例
 */
import axios from 'axios';
import type { Method, RawAxiosRequestHeaders, ResponseType } from 'axios';
import { TRequestData } from '@/types/Request';
import router from '@/router';
import { EleMessage } from 'ele-admin-plus';
import { getUserInfo, removeToken } from './token-util';

/** 创建axios实例 */
const service = axios.create({
  baseURL: import.meta.env.VITE_API_URL, // API_BASE_URL
  timeout: 200000
});

/**
 * 添加请求拦截器
 */
service.interceptors.request.use(
  (config) => {
    if (config.headers) {
      const session = getUserInfo();
      if (session && session.userInfo) {
        config.headers.token = session.userInfo.signToken;
      }
    }
    return config;
  },
  (err) => {
    return Promise.reject(err);
  }
);

/**
 * 添加响应拦截器
 */
service.interceptors.response.use(
  (res) => {
    const { data }: { data: TRequestData<any> } = res;
    switch (data.code) {
      case 103:
        EleMessage.error(data.message);
        removeToken();
        router.replace('/');
        break;
      case 105:
        // 已在其它地方登录,请下线
        EleMessage.error(data.message);
        removeToken();
        router.replace('/');
        break;
      case 1001:
        EleMessage.error(data.message);
        break;
      case 500:
        EleMessage.error(data.message);
    }
    return res.data;
  },
  (err) => {
    switch (err.response.status) {
      case 404:
        EleMessage.error(err.response.config.url + '接口资源请求失败404');
        break;
      case 500:
        if (err.response.data.code === -1) {
          EleMessage.warning(err.response.data.message);
        } else {
          EleMessage.error(err.response.config.url + '接口资源请求失败500');
        }
        break;
      case 400:
        EleMessage.error(
          err.response.config.url +
            '接口资源请求失败400,失败原因:' +
            err.response.data.message
        );
    }
    return Promise.reject(err);
  }
);

/**
 *
 * @param url 请求url地址
 * @param method  请求方法
 * @param submitData 可选,请求参数
 * @param headers 可选,设置请求头
 * @returns 返回一个 Axios 请求，该请求返回一个 Promise，解析为请求的响应数据
 */
const request = <T>(
  url: string,
  method: Method,
  submitData?: object,
  headers?: RawAxiosRequestHeaders,
  responseType?: ResponseType
) => {
  return service.request<T, TRequestData<T>>({
    url,
    method,
    [method.toLowerCase() === 'get' ? 'params' : 'data']: submitData,
    headers,
    responseType
  });
};

export { service, request };

/**
 * 创建api请求地址
 * @param prefix api前缀
 * @param apiObj 接口请求地址对象
 * @returns 拼接prefix前缀后的地址对接
 */
export const createApiUrl = <T extends Record<string, string>>(
  prefix: string,
  apiObj: T
): { [K in keyof T]: string } => {
  const result: Partial<{ [K in keyof T]: string }> = {};
  for (const key in apiObj) {
    if (apiObj.hasOwnProperty(key)) {
      result[key] = `${prefix}${apiObj[key]}`;
    }
  }
  return result as { [K in keyof T]: string };
};
